"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const fs = require("fs");
const Ajv = require("ajv");
const util_1 = require("./util");
const schema_1 = require("./schema");
const ajv = new Ajv(); // options can be passed, e.g. {allErrors: true}
const ajvValidator = ajv.compile(schema_1.configSchema);
class Command {
    constructor(args) {
        this.input = args.input;
        this.args = args.flags;
        this.configPath = args.flags.configPath || path.join(process.cwd(), "projects.yml");
        this.config = this.getCommandConfig();
        this.config.baseDir = args.flags.baseDir || this.config.baseDir;
    }
    eachProject(fn, options = {}) {
        let value = 0;
        const { projects, baseDir } = this.config;
        const filteredProjects = options.filter ? projects.filter(options.filter) : projects;
        for (const project of filteredProjects) {
            value++;
            const projectDir = path.join(baseDir, project.package);
            fn(projectDir, project, { ...this.args, countOf: [value, filteredProjects.length] });
        }
    }
    getCommandConfig() {
        if (!fs.existsSync(this.configPath)) {
            throw Error(`config file doesnt exist:\n ${this.configPath}`);
        }
        const config = util_1.getConfig(this.configPath);
        this.validateConfig(config);
        return config;
    }
    validateConfig(config) {
        const valid = ajvValidator(config);
        if (!valid) {
            util_1.logger.error(JSON.stringify(ajvValidator.errors, null, 2));
            throw Error(`Invalid configuration file:`);
        }
        return valid;
    }
}
exports.Command = Command;
