"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const meow = require("meow");
const chalk_1 = require("chalk");
const util_1 = require("./util");
const Setup_1 = require("./commands/Setup");
const Relink_1 = require("./commands/Relink");
const Rebuild_1 = require("./commands/Rebuild");
const defaultFlags = {
    buildDir: {
        name: "buildDir",
        alias: "t"
    },
    configPath: {
        name: "configPath",
        alias: "c"
    },
    help: {
        name: "help",
        alias: "h"
    }
};
const meowConfig = {
    flags: defaultFlags,
    autoHelp: false,
    argv: process.argv.slice(3)
};
const defaultHelpOptions = `
Global Options:

--base-dir, -t
  specify which build directory to target.
  overrides config file value

--config-path, -c
  specify path to config file
  ./projects.yml by default

--help, -h
  help with the particular command
`;
const commands = {
    setup: Setup_1.Setup,
    relink: Relink_1.Relink,
    rebuild: Rebuild_1.Rebuild
};
const listAvailableCommands = () => {
    return Object.values(commands)
        .filter(cmd => cmd.commandName)
        .reduce((list, cmd) => `${list}\n - yarn-compose ${cmd.commandName}`, "");
};
(async () => {
    try {
        const [, , commandArg] = process.argv;
        if (!commandArg) {
            throw Error(`No command provided. Try: ${listAvailableCommands()}`);
        }
        if (!commands[commandArg]) {
            throw Error(`Command ${chalk_1.default.whiteBright.bold(process.argv[2])} does not exist.\nTry: ${listAvailableCommands()}`);
        }
        const SelectedCmd = commands[commandArg];
        util_1.logger.meta(SelectedCmd.commandName);
        if (SelectedCmd.additionalFlags) {
            meowConfig.flags = Object.assign(meowConfig.flags, SelectedCmd.additionalFlags);
        }
        const otherArgs = meow(SelectedCmd.commandHelp, meowConfig);
        if (otherArgs.flags.help || !commandArg) {
            util_1.logger.help(SelectedCmd.commandHelp + "\n" + defaultHelpOptions);
            process.exit(0);
        }
        const Cmd = new SelectedCmd(otherArgs);
        await Cmd.run();
    }
    catch (err) {
        util_1.logger.error(err);
        process.exit(1);
    }
})();
